#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2024-2025 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Wrapper for libuuu."""

from .__version__ import __version__ as version
from .libuuu import LibUUU, UUUNotifyCallback, UUUShowConfig, UUUState

__author__ = """NXP"""
__version__ = version
__all__ = ["LibUUU", "UUUNotifyCallback", "UUUShowConfig", "UUUState"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2025 NXP
#
# SPDX-License-Identifier: BSD-3-Clause
"""Custom build backend that wraps setuptools and forces platform wheels."""

import os
from typing import Any, Tuple

from setuptools.build_meta import build_editable as _build_editable
from setuptools.build_meta import build_sdist as _build_sdist
from setuptools.build_meta import build_wheel as _build_wheel
from setuptools.dist import Distribution
from wheel.bdist_wheel import bdist_wheel


def patch() -> None:
    """Patch the bdist wheel build."""
    # Apply the monkey patch when the module is imported
    original_has_ext_modules = getattr(Distribution, "has_ext_modules", None)
    if original_has_ext_modules is not None:
        Distribution.has_ext_modules = lambda self: True

    # Also patch the is_pure method to return False
    original_is_pure = getattr(Distribution, "is_pure", None)
    if original_is_pure is not None:
        Distribution.is_pure = lambda self: False

    # Override the wheel tag generation

    original_get_tag = getattr(bdist_wheel, "get_tag", None)
    if original_get_tag is not None:

        def tag(self: Any) -> Tuple[str, str, str]:
            _, _, platform_tag = original_get_tag(self)
            return ("py3", "none", platform_tag)

        bdist_wheel.get_tag = tag

    print("Custom build backend loaded: Platform-specific wheels enabled")


def validate_native_libraries() -> None:
    """Validate that native libraries exist in the expected locations."""
    lib_dir = os.path.join(os.path.dirname(__file__), "libuuu", "lib")

    if os.path.exists(lib_dir):
        print(f"Native libraries found in: {lib_dir}")
        # List found libraries
        for root, _, files in os.walk(lib_dir):
            for file in files:
                if file.endswith((".dll", ".so", ".dylib")):
                    print(f"  - {os.path.relpath(os.path.join(root, file), lib_dir)}")
    else:
        print(f"Warning: No native libraries found in {lib_dir}")


def build_wheel(wheel_directory, config_settings=None, metadata_directory=None) -> str:  # type: ignore
    """Build wheel."""
    patch()
    validate_native_libraries()
    return _build_wheel(wheel_directory, config_settings, metadata_directory)


def build_sdist(sdist_directory, config_settings=None) -> str:  # type: ignore
    """Build source."""
    patch()
    validate_native_libraries()
    return _build_sdist(sdist_directory, config_settings)


def build_editable(wheel_directory, config_settings=None, metadata_directory=None) -> str:  # type: ignore
    """Build editable."""
    patch()
    validate_native_libraries()
    return _build_editable(wheel_directory, config_settings, metadata_directory)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2024-2025 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Main module for libuuu wrapper."""

import logging
import pathlib
import platform
from ctypes import (
    CDLL,
    CFUNCTYPE,
    POINTER,
    Structure,
    Union,
    c_char_p,
    c_int,
    c_size_t,
    c_ulonglong,
    c_ushort,
    c_void_p,
)
from dataclasses import dataclass
from enum import Enum, auto


class UUUNotifyType(Enum):
    """An enumeration to represent the different types of notifications from uuu."""

    NOTIFY_CMD_TOTAL = 0
    NOTIFY_CMD_START = auto()
    NOTIFY_CMD_END = auto()

    NOTIFY_CMD_INDEX = auto()

    NOTIFY_CMD_INFO = auto()

    NOTIFY_PHASE_TOTAL = auto()
    NOTIFY_PHASE_INDEX = auto()

    NOTIFY_TRANS_SIZE = auto()
    NOTIFY_TRANS_POS = auto()

    NOTIFY_WAIT_FOR = auto()
    NOTIFY_DEV_ATTACH = auto()

    NOTIFY_DECOMPRESS_START = auto()
    NOTIFY_DECOMPRESS_SIZE = auto()
    NOTIFY_DECOMPRESS_POS = auto()

    NOTIFY_DOWNLOAD_START = auto()
    NOTIFY_DOWNLOAD_END = auto()
    NOTIFY_THREAD_EXIT = auto()

    NOTIFY_DONE = auto()


@dataclass
class UUUNotifyDataUnion(Union):
    """A union to store data from a uuu notification."""

    _fields_ = [
        ("status", c_int),
        ("index", c_size_t),
        ("total", c_size_t),
        ("str", c_char_p),
    ]


@dataclass
class UUUNotifyStruct(Structure):
    """A class to store a notification from uuu."""

    _fields_ = [
        ("_type", c_int),
        ("id", c_ulonglong),
        ("timestamp", c_ulonglong),
        ("response", UUUNotifyDataUnion),
    ]

    @property
    def type(self) -> UUUNotifyType:
        """Return type attribute as an enum.

        :return UUUNotifyType enum
        """
        return UUUNotifyType(self._type)


@dataclass
class UUUCommandResponse(Structure):
    """A class to store the response from a uuu command."""

    _fields_ = [
        ("value", c_char_p),
    ]


UUUNotifyCallback = CFUNCTYPE(c_int, UUUNotifyStruct, POINTER(c_void_p))

UUUShowConfig = CFUNCTYPE(
    c_int,
    c_char_p,
    c_char_p,
    c_char_p,
    c_ushort,
    c_ushort,
    c_ushort,
    c_ushort,
    c_void_p,
)


@dataclass
class UUUState:
    """A class to store the state of a uuu library command call."""

    # pylint: disable=too-many-instance-attributes
    def __init__(self) -> None:
        """Constructor of UUUState class."""
        self.logger = logging.getLogger("UUUState")
        self.cmd: str = ""
        self.dev: str = ""

        self.waiting: bool = False
        self.done: bool = False
        self.error: bool = False

        self.cmd_status: bool = False
        self.cmd_done: bool = False

        self.cmd_end: int = 0
        self.cmd_pos: int = 0
        self.cmd_start: int = 0

        self.cmd_total: int = 0
        self.cmd_index: int = 0

        self.trans_pos: int = 0
        self.trans_size: int = 0

    def update(self, struct: UUUNotifyStruct) -> None:
        """Update the state with a notification from uuu.

        :param struct: A UUUNotifyStruct object
        """
        self.waiting = struct.type == UUUNotifyType.NOTIFY_WAIT_FOR
        self.done = struct.type == UUUNotifyType.NOTIFY_DONE

        if struct.type == UUUNotifyType.NOTIFY_CMD_TOTAL:
            self.cmd_total = struct.response.total
        elif struct.type == UUUNotifyType.NOTIFY_CMD_START:
            self.cmd = struct.response.str
            self.cmd_pos = 0
            self.cmd_start = struct.timestamp
        elif struct.type == UUUNotifyType.NOTIFY_CMD_END:
            status = struct.response.status
            self.done = True
            self.error = status != 0
            self.cmd_end = struct.timestamp
            if status != 0:
                self.logger.warning(f"Command {self.cmd} failed")
        elif struct.type == UUUNotifyType.NOTIFY_DEV_ATTACH:
            self.dev = struct.response.str
            self.done = False
            self.error = False
        elif struct.type == UUUNotifyType.NOTIFY_TRANS_SIZE:
            self.trans_size = struct.response.total
        elif struct.type == UUUNotifyType.NOTIFY_TRANS_POS:
            self.trans_pos = struct.response.total

        self.logger.debug(f"{self.cmd=},{self.dev=},{self.waiting=}")


def get_platform_info() -> tuple[str, str]:
    """Get platform and architecture information."""
    system = platform.system().lower()
    machine = platform.machine().lower()

    # Normalize architecture names
    arch_mapping = {
        "x86_64": "x86_64",
        "amd64": "x86_64",
        "aarch64": "aarch64",
        "arm64": "arm64",
    }

    arch = arch_mapping.get(machine, machine)
    if system == "linux" and arch == "arm64":
        # For Linux on ARM64, we can use the 'aarch64' architecture name
        arch = "aarch64"

    return system, arch


def get_dll_path() -> str:
    """Return path to the appropriate shared library based on platform and architecture."""
    system, arch = get_platform_info()

    base_path = pathlib.Path(__file__).parent / "lib"

    if system == "windows":
        lib_name = "libuuu.dll"
        lib_path = base_path / "windows" / arch / lib_name
    elif system == "darwin":
        lib_name = "libuuu.dylib"
        # macOS uses universal binary
        lib_path = base_path / "darwin" / arch / lib_name
    else:  # Linux and other Unix-like systems
        lib_name = "libuuu.so"
        lib_path = base_path / "linux" / arch / lib_name

    return str(lib_path)


@UUUNotifyCallback
def _default_notify_callback(struct: UUUNotifyStruct, data) -> int:  # type: ignore
    """A default callback function that stores the response in a class variable.

    :param struct: A UUUNotifyStruct object
    :param data: A pointer to data, here it is not used
    """
    # pylint: disable=unused-argument
    LibUUU._state.update(struct)
    if struct.type == UUUNotifyType.NOTIFY_CMD_INFO:
        LibUUU._response.value += bytes(struct.response.str)
    return 1 if LibUUU._state.error else 0


class LibUUU:
    """Wrapper for the libuuu library."""

    DLL = get_dll_path()
    NULL = POINTER(c_void_p)()

    _response = UUUCommandResponse()
    _state = UUUState()

    def __init__(self) -> None:
        """Initialize the library and register the default notify callback function."""
        self._response.value = b""
        self.lib = CDLL(self.DLL, mode=1)
        self.register_notify_callback(_default_notify_callback, self.NULL)

    def set_wait_timeout(self, timeout: int) -> int:
        """Set the wait timeout for uuu in seconds."""
        return self.lib.uuu_set_wait_timeout(c_int(timeout))

    def set_wait_next_timeout(self, timeout: int) -> int:
        """Set the wait next timeout for uuu in seconds."""
        return self.lib.uuu_set_wait_next_timeout(c_int(timeout))

    def set_poll_period(self, period: int) -> int:
        """Set the poll period for uuu in milliseconds."""
        return self.lib.uuu_set_poll_period(c_int(period))

    def set_debug_level(self, level: int) -> int:
        """Set the debug level for uuu.

        :param level: The debug level [15:0] for libusb, [31:16] for uuu
        """
        return self.lib.uuu_set_debug_level(c_int(level))

    def set_small_mem(self, size: int) -> int:
        """Disable small memory mode and buffer all data.

        :param size: The size of the buffer in bytes
        """
        return self.lib.uuu_set_small_mem(c_int(size))

    def get_version_string(self) -> str:
        """Get the version of uuu."""
        self.lib.uuu_get_version_string.restype = c_char_p
        return self.lib.uuu_get_version_string().decode()

    def get_version(self) -> int:
        """Get the version of uuu.

        :return: The version of uuu bits represent version in format [31:24].[23:12].[11:0]
        """
        return self.lib.uuu_get_version()

    def get_last_error_string(self) -> str:
        """Get the last error message from uuu."""
        self.lib.uuu_get_last_err_string.restype = c_char_p
        return self.lib.uuu_get_last_err_string().decode()

    def get_last_error(self) -> int:
        """Get the last error code from uuu."""
        return self.lib.uuu_get_last_err()

    def run_cmd(self, cmd: str, dry: bool = False) -> int:
        """Run a uuu command.

        :param cmd: The command to run
        :param dry: If set to False command will be executed, otherwise its a dry run
        :return: 0 if success
        """
        # pylint: disable=attribute-defined-outside-init
        self._response.value = b""
        return self.lib.uuu_run_cmd(c_char_p(str.encode(cmd)), c_int(1 if dry else 0))

    def register_notify_callback(self, callback: UUUNotifyCallback, data: any) -> int:  # type: ignore
        """Register a callback function to receive notifications from uuu.

        :param callback: A function that will be called when uuu sends a notification
        :param data: A pointer to data that will be passed to the callback function
        :return: 0 on success, otherwise failure
        """
        return self.lib.uuu_register_notify_callback(callback, data)

    def unregister_notify_callback(self, callback: UUUNotifyCallback) -> int:  # type: ignore
        """Unregister the callback function."""
        return self.lib.uuu_unregister_notify_callback(callback)

    @property
    def response(self) -> bytes:
        """Get the response from the last uuu command."""
        return self._response.value



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
## Copyright 2025 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Basic tests for libuuu wrapper."""

import pytest

from libuuu import LibUUU
from libuuu.libuuu import UUUNotifyType, UUUState, get_dll_path, get_platform_info


def test_check_dlopen_works() -> None:
    """Test that the library can be loaded."""
    lib = LibUUU()
    assert lib is not None


def test_get_last_error_string() -> None:
    """Test getting last error string."""
    lib = LibUUU()
    error_str = lib.get_last_error_string()
    assert isinstance(error_str, str)


def test_get_last_error_code() -> None:
    """Test getting last error code."""
    lib = LibUUU()
    error_code = lib.get_last_error()
    assert isinstance(error_code, int)


def test_set_wait_timeout() -> None:
    """Test setting wait timeout."""
    lib = LibUUU()
    result = lib.set_wait_timeout(30)
    assert isinstance(result, int)


def test_set_wait_next_timeout() -> None:
    """Test setting wait next timeout."""
    lib = LibUUU()
    result = lib.set_wait_next_timeout(5)
    assert isinstance(result, int)


def test_set_poll_period() -> None:
    """Test setting poll period."""
    lib = LibUUU()
    result = lib.set_poll_period(1000)
    assert isinstance(result, int)


def test_set_debug_level() -> None:
    """Test setting debug level."""
    lib = LibUUU()
    result = lib.set_debug_level(0)
    assert isinstance(result, int)


def test_set_small_mem() -> None:
    """Test setting small memory mode."""
    lib = LibUUU()
    result = lib.set_small_mem(1024 * 1024)  # 1MB
    assert isinstance(result, int)


def test_run_cmd_dry_run() -> None:
    """Test running a command in dry run mode."""
    lib = LibUUU()
    # Use a simple help command for dry run
    result = lib.run_cmd("help", dry=True)
    assert isinstance(result, int)


def test_response_property() -> None:
    """Test that response property returns bytes."""
    lib = LibUUU()
    response = lib.response
    assert isinstance(response, bytes)


def test_get_platform_info() -> None:
    """Test getting platform information."""
    system, arch = get_platform_info()
    assert isinstance(system, str)
    assert isinstance(arch, str)
    assert len(system) > 0
    assert len(arch) > 0
    # Check that we get expected values
    assert system in ["windows", "darwin", "linux"]
    assert arch in ["x86_64", "aarch64", "armv7l", "arm64"]


def test_get_dll_path() -> None:
    """Test getting DLL path."""
    dll_path = get_dll_path()
    assert isinstance(dll_path, str)
    assert len(dll_path) > 0
    # Check that path contains expected library extension
    assert any(ext in dll_path for ext in [".dll", ".so", ".dylib"])


def test_uuu_notify_type_enum() -> None:
    """Test UUUNotifyType enum values."""
    # Test that enum has expected values
    assert UUUNotifyType.NOTIFY_CMD_TOTAL.value == 0
    assert UUUNotifyType.NOTIFY_CMD_START.value == 1
    assert UUUNotifyType.NOTIFY_CMD_END.value == 2
    assert UUUNotifyType.NOTIFY_DONE.value == 17


def test_uuu_state_initialization() -> None:
    """Test UUUState class initialization."""
    state = UUUState()
    assert state.cmd == ""
    assert state.dev == ""
    assert state.waiting is False
    assert state.done is False
    assert state.error is False
    assert state.cmd_status is False
    assert state.cmd_done is False
    assert state.cmd_total == 0
    assert state.cmd_index == 0
    assert state.trans_pos == 0
    assert state.trans_size == 0


def test_library_constants() -> None:
    """Test library constants."""
    lib = LibUUU()
    assert hasattr(lib, "DLL")
    assert hasattr(lib, "NULL")
    assert isinstance(lib.DLL, str)


@pytest.mark.parametrize("timeout_value", [1, 10, 30, 60])
def test_set_wait_timeout_various_values(timeout_value: int) -> None:
    """Test setting various timeout values."""
    lib = LibUUU()
    result = lib.set_wait_timeout(timeout_value)
    assert isinstance(result, int)


@pytest.mark.parametrize("debug_level", [0, 1, 15, 65535])
def test_set_debug_level_various_values(debug_level: int) -> None:
    """Test setting various debug levels."""
    lib = LibUUU()
    result = lib.set_debug_level(debug_level)
    assert isinstance(result, int)


def test_error_handling_invalid_command() -> None:
    """Test error handling with invalid command."""
    lib = LibUUU()
    # Try to run an invalid command in dry run mode
    result = lib.run_cmd("invalid_command_xyz", dry=True)
    # Should return an error code (non-zero) but not crash
    assert isinstance(result, int)


def test_get_version() -> None:
    """Test getting library version as an integer."""
    lib = LibUUU()
    ver = lib.get_version()
    assert isinstance(ver, int)


def test_get_version_string() -> None:
    """Test getting library version as a string."""
    lib = LibUUU()
    ver = lib.get_version_string()
    assert isinstance(ver, str)
